class Email < Sequel::Model($db)
  many_to_one :mailinglist

  def before_create
    self.created_at ||= Time.now
    super
  end

  def before_save
    self.updated_at ||= Time.now
    super
  end

  module Permissions
    READ  = 1
    WRITE = 2
    ADMIN = 4
  end

  # @param name
  # @param email
  # @param mailinglist required if mailinglist.nil?
  # @param mailinglist_id required if mailinglist.nil?
  def self.register(name:, email:, mailinglist: nil, mailinglist_id: nil)
    mailinglist = Mailinglist.first(id: mailinglist_id) if mailinglist.nil?
    permissions =
      if mailinglist.strategy == "free"
        Permissions::READ | Permissions::WRITE
      elsif mailinglist.strategy == "validated"
        0
      elsif mailinglist.strategy == "closed"
        return nil # not valid
      end

    new(
      name: name,
      email: email,
      mailinglist_id: mailinglist.id,
      permissions: permissions,
    )
  end

  def self.unregister(mailinglist:, email:)
    Email.where(mailinglist:, email:).delete
  end
end
